unit dmserver;

{$mode ObjFPC}

interface

uses
  Classes, SysUtils, jsondataset, sqldbrestdataset, services, fprpcclient;

type
  TUserInfo = record
    Login : String;
    ID : NativeInt;
  end;

  { TServerModule }

  TServerModule = class(TDataModule)
    RPCClient: TPas2jsRPCClient;
    RestConnection: TSQLDBRestConnection;
    procedure DataModuleCreate(Sender: TObject);
  private
    FOnInvalidLogin: TNotifyEvent;
    FOnLogin: TNotifyEvent;
    FUserInfo: TUserInfo;
    FUserService: TUserService;
  public
    Procedure DoLogin(const aUserName,aPassword : String);
    Property UserService : TUserService Read FUserService;
    Property UserInfo : TUserInfo Read FUserInfo;
    Property OnInvalidLogin : TNotifyEvent Read FOnInvalidLogin Write  FOnInvalidLogin;
    Property OnLogin : TNotifyEvent Read FOnLogin Write FOnLogin;
  end;

var
  Server : TServerModule;

implementation

{$R *.lfm}

{ TServerModule }

Const
  ServerURL = 'http://localhost:8080/';

procedure TServerModule.DataModuleCreate(Sender: TObject);

begin
  FUserService:=TUserService.Create(Self);
  FUserService.RPCClient:=RPCclient;
  RPCClient.URL:=ServerURL+'RPC/';
  RestConnection.BaseURL:=ServerURL+'REST/';
end;

procedure TServerModule.DoLogin(const aUserName, aPassword: String);

  procedure DoOK(aResult: NativeInt);
  begin
    FUserInfo.Login:=aUserName;
    FUserInfo.ID:=aResult;
    if Assigned(FOnLogin) then
      FonLogin(Self);
  end;

  procedure DoFail(Sender: TObject; const aError: TRPCError);
  begin
    FUserInfo.Login:='';
    FUserInfo.ID:=-1;
    If Assigned(FOnInvalidLogin) then
      FOnInvalidLogin(Self);
  end;

begin
  UserService.Login(aUserName,aPassword,@DoOK,@DoFail);
end;

end.

