unit frmarticle;

{$mode ObjFPC}

interface

uses
  SysUtils, Classes, Rtl.HTMLActions, htmlfragment, jsondataset, types,
  sqldbrestdataset, DB, dbwebwidget, webwidget, webrouter, frmbase, Web;

type
  THTMLFragment = TBaseForm;

  { TArticleForm }

  TArticleForm = class(THTMLFragment)
    btnAddComment: THTMLElementAction;
    btnSave: THTMLElementAction;
    divArticle: THTMLElementAction;
    divArticleBlock: THTMLElementAction;
    divComments: THTMLElementAction;
    divEditTitle: THTMLElementAction;
    divExistingComments: THTMLElementAction;
    divNoArticles: THTMLElementAction;
    divTitle: THTMLElementAction;
    editcomment: THTMLElementAction;
    edtTitle: THTMLElementAction;
    lblAuthor: THTMLElementAction;
    lblDate: THTMLElementAction;
    lblTitle: THTMLElementAction;
    ltwComments: TDBLoopTemplateWidget;
    dsArticle: TDataSource;
    dsComments: TDataSource;
    alForm: THTMLElementActionList;
    rdsArticle: TSQLDBRestDataset;
    rdsArticlea_author_fk: TLargeintField;
    rdsArticlea_created_on: TJSONDateTimeField;
    rdsArticlea_id: TLargeintField;
    rdsArticlea_text: TStringField;
    rdsArticlea_title: TStringField;
    rdsArticleu_firstname: TStringField;
    rdsArticleu_lastname: TStringField;
    rdsComments: TSQLDBRestDataset;
    rdsCommentsc_article_fk: TLargeintField;
    rdsCommentsc_author_fk: TLargeintField;
    rdsCommentsc_created_on: TJSONDateTimeField;
    rdsCommentsc_id: TLargeintField;
    rdsCommentsc_text: TStringField;
    rdsCommentsu_firstname: TStringField;
    rdsCommentsu_lastname: TStringField;
    procedure btnAddCommentExecute(Sender: TObject; Event: TJSEvent);
    procedure btnSaveExecute(Sender: TObject; Event: TJSEvent);
    procedure rdsArticleAfterOpen(DataSet: TDataSet);
  private
    FarticleID : NativeInt;
    class procedure HideEl(aEl: THTMLElementAction);
    procedure SetupForEdit(IsEditing: Boolean);
    procedure ShowData(Dataset: TDataset);
    class procedure ShowEl(aEl: THTMLElementAction);
    procedure SaveData(Dataset: TDataset);

  public
    Procedure ShowRoute(Const aURL : String; aRoute : TRoute; aParams : TStrings); override;
    class function FormRoutes : TStringDynArray; override;

  end;

var
  ArticleForm: TArticleForm;

implementation

uses dmServer, libtinyeditor;

{$R *.lfm}

{ TArticleForm }

class procedure TArticleForm.HideEl(aEl: THTMLElementAction);

begin
  aEl.addClass('is-hidden');
end;

class procedure TArticleForm.ShowEl(aEl: THTMLElementAction);

begin
  aEl.RemoveClass('is-hidden');
end;

procedure TArticleForm.SetupForEdit(IsEditing : Boolean);

begin
  if IsEditing then
    begin
    HideEl(divComments);
    HideEl(divTitle);
    ShowEl(divEditTitle);
    tinyEditor.transformToEditor(divArticle.Element);
    ShowEl(btnSave);
    end
  else
    begin
    HideEl(divEditTitle);
    ShowEl(divComments);
    (btnAddComment.Element as TJSHTMLButtonElement).disabled:=(Server.UserInfo.ID<=0);
    if not (btnAddComment.Element as TJSHTMLButtonElement).disabled then
      begin
      editcomment.Element.style.setProperty('min-height','10em;');
      tinyEditor.transformToEditor(editcomment.element);
      end;
    end;
end;

procedure TArticleForm.ShowData(Dataset : TDataset);

var
  S : string;

begin
  With Dataset do
    begin
    S:=FieldByName('u_firstname').AsString + ' '
       + FieldByName('u_lastname').AsString;
    lblAuthor.Value:=S;
    S:=FormatDateTime('yyyy-mm-dd hh:nn',
                      FieldByName('a_created_on').AsDateTime);
    lblDate.Value:=S;
    S:=FieldByName('a_title').AsString;
    edtTitle.value:=S;
    lblTitle.Value:=S;
    S:=FieldByName('a_text').AsString;
    divArticle.Element.InnerHtml:=S;
    end;
end;


procedure TArticleForm.rdsArticleAfterOpen(DataSet: TDataSet);

Var
  IsEdit : Boolean;
  S : String;

begin
  if Dataset.IsEmpty then
    begin
    if FarticleID>0 then
      begin
      HideEl(divArticle);
      HideEl(divTitle);
      HideEl(divEditTitle);
      HideEl(divComments);
      ShowEl(divNoArticles);
      end
    else
      begin
      lblAuthor.Value:=Server.UserInfo.Login;
      S:=FormatDateTime('yyyy-mm-dd hh:nn',Now);
      lblDate.Value:=S;
      edtTitle.value:='';
      divArticle.Value:='';
      SetupForEdit(True);
      Dataset.Append;
      end;
    end
  else
    begin
    isEdit:=Dataset.FieldByname('a_author_fk').AsInteger=Server.UserInfo.ID;
    SetupForEdit(isEdit);
    ShowData(Dataset);
    if IsEdit then
      Dataset.Edit;
    end;
  With Dataset.FieldByname('a_id') do
    ProviderFlags:=ProviderFlags+[pfInKey];
end;


procedure TArticleForm.SaveData(Dataset : TDataset);

begin
  rdsArticlea_created_on.AsDateTime:=Now;
  rdsArticlea_text.AsString:=divArticle.Element.InnerHTML;
  rdsArticlea_title.AsString:=edtTitle.Value;
  rdsArticlea_author_fk.AsInteger:=Server.UserInfo.ID;
end;


procedure TArticleForm.btnSaveExecute(Sender: TObject; Event: TJSEvent);
begin
  SaveData(rdsArticle);
  rdsArticle.Post;
  rdsArticle.ApplyUpdates;
  rdsArticle.Edit;
end;

procedure TArticleForm.btnAddCommentExecute(Sender: TObject; Event: TJSEvent);
begin
  With rdsComments do
    begin
    Append;
    rdsCommentsc_text.AsString:=editcomment.Element.innerHTML;
    rdsCommentsc_created_on.AsDatetime:=Now;
    rdsCommentsc_author_fk.AsDateTime:=Server.UserInfo.ID;
    Post;
    ApplyUpdates;
    end;
end;

procedure TArticleForm.ShowRoute(const aURL: String; aRoute: TRoute;
  aParams: TStrings);
Var
  aID : String;

begin
  Inherited;
  aID:=aParams.Values['ArticleID'];
  if SameText(aID,'New') then
    begin
    FArticleID:=-1;
    rdsArticle.ParamByName('a_author_fk').asInteger:=-1;
    rdsArticle.ParamByName('a_author_fk').Enabled:=True;
    end
  else
    begin
    FArticleID:=StrToIntDef(aID,-1);
    rdsArticle.ResourceID:=IntToStr(FArticleID);
    end;
  rdsArticle.Load();
  if (FArticleID>0) then
    begin
    rdsComments.Params[0].asLargeInt:=FArticleID;
    rdsComments.Load();
    end
end;

class function TArticleForm.FormRoutes: TStringDynArray;
begin
  Result:=['/article/:ArticleID'];
end;

end.

